@@ -12,7 +12,9 @@ Will call into win32 commands as needed when needed
 """
 import logging
 import re
+
 from edk2toollib.utility_functions import GetHostInfo
+
 try:
     # try to import windows types from winDLL
     import ctypes
@@ -34,7 +36,7 @@ except (AttributeError, ImportError):
 else:
     # if we don't raise an exception when we import windows types
     # then execute this but don't catch an exception if raised
-    from ctypes import byref, Structure
+    from ctypes import Structure, byref
 
     # inspired by https://github.com/tartley/colorama/
     class CONSOLE_SCREEN_BUFFER_INFO(Structure):
@@ -166,7 +168,7 @@ class AnsiColor(object):
     @classmethod
     def __contains__(self, item):
         """Verifies we contain the color."""
-        if type(item) is str and hasattr(self, item):
+        if isinstance(item, str) and hasattr(self, item):
             return True
         # check if we contain the color number
         for attr_name in dir(self):
@@ -210,7 +212,7 @@ class ColoredFormatter(logging.Formatter):
             # otherwise color the wholes message
             else:
                 levelname_color = get_ansi_string(ColoredFormatter.COLORS[levelname]) + levelname
-                record.msg += get_ansi_string()
+                record.msg = str(org_message) + get_ansi_string()
             record.levelname = levelname_color
 
         if self.use_azure and levelname in ColoredFormatter.AZURE_COLORS:
