@@ -8,6 +8,7 @@
 """Data model for the EDK II DSC."""
 # There will be some overlap between the objects for DSC files and FDF files
 import collections
+
 DEFAULT_SECTION_TYPE = "COMMON"
 
 
@@ -70,9 +71,9 @@ class dsc_dict(collections.OrderedDict):
             raise ValueError(f"Cannot add {type(key)} to restricted set: {self._allowed_key_classes}")
 
         if len(self._allowed_value_classes) > 0:
-            if type(val) == set and len(val) == 0:  # if it's an empty set, convert it to a dsc_set
+            if isinstance(val, set) and len(val) == 0:  # if it's an empty set, convert it to a dsc_set
                 val = dsc_set(allowed_classes=self._allowed_value_classes)
-            if type(val) == dsc_set:
+            if isinstance(val, dsc_set):
                 if val._allowed_classes != self._allowed_value_classes:
                     raise ValueError(
                         f"Cannot add set:{val._allowed_classes} to restricted dict: {self._allowed_value_classes}")
@@ -554,7 +555,7 @@ class pcd_variable(pcd):
         self.var_offset = var_offset
         self.default = default
 
-        if type(attributes) is str:
+        if isinstance(attributes, str):
             attributes = attributes.split(",")
         attributes = [str(x).upper().strip() for x in attributes]
         if any([x not in pcd_variable_attributes for x in attributes]):
