@@ -9,8 +9,8 @@
 """Simple list of GuidListEntry objects parsed from edk2 specific files."""
 import logging
 import os
-from edk2toollib.gitignore_parser import parse_gitignore_lines
 
+from edk2toollib.gitignore_parser import parse_gitignore_lines
 from edk2toollib.uefi.edk2.parsers.dec_parser import DecParser
 from edk2toollib.uefi.edk2.parsers.inf_parser import InfParser
 
@@ -113,7 +113,7 @@ class GuidList():
 
         try:
             results.append(GuidListEntry(dec.Dict["PACKAGE_NAME"], dec.Dict["PACKAGE_GUID"], filename))
-        except:
+        except Exception:
             logging.warning("Failed to find Package Guid from dec file: " + filename)
         return results
 
@@ -131,6 +131,6 @@ class GuidList():
         inf.ParseFile(filename)
         try:
             return [GuidListEntry(inf.Dict["BASE_NAME"], inf.Dict["FILE_GUID"].upper(), filename)]
-        except:
+        except Exception:
             logging.warning("Failed to find info from INF file: " + filename)
         return []
