@@ -6,11 +6,12 @@
 # SPDX-License-Identifier: BSD-2-Clause-Patent
 ##
 """Code to support parsing EDK2 files."""
-import os
 import logging
-from edk2toollib.uefi.edk2 import path_utilities
+import os
 from warnings import warn
 
+from edk2toollib.uefi.edk2 import path_utilities
+
 
 class BaseParser(object):
     """Base Parser for other parser objects.
@@ -166,7 +167,7 @@ class BaseParser(object):
         # If that fails, check in every possible Pkg path.
         if self._Edk2PathUtil is not None:
             target_path = os.path.join(*p)
-            Path = self._Edk2PathUtil.GetAbsolutePathOnThisSystemFromEdk2RelativePath(target_path, False)
+            Path = self._Edk2PathUtil.GetAbsolutePathOnThisSystemFromEdk2RelativePath(target_path, log_errors=False)
             if Path is not None:
                 return Path
 
@@ -239,12 +240,18 @@ class BaseParser(object):
             return (ivalue != ivalue2) and (value != value2)
 
         # check to make sure we only have digits from here on out
-        if not isinstance(value, int) and not str.isdigit(value):
+        if (isinstance(value, str) and value.upper() in ["TRUE", "FALSE"]) \
+            or (isinstance(value, str) and value2.upper() in ["TRUE", "FALSE"]):
+            self.Logger.error(f"Invalid comparison: {value} {cond} {value2}")
+            self.Logger.debug(f"Invalid comparison: {value} {cond} {value2}")
+            raise ValueError("Invalid comparison")
+
+        if not isinstance(ivalue, int) and not str.isdigit(value):
             self.Logger.error(f"{self.__class__}: Unknown value: {value} {ivalue.__class__}")
             self.Logger.debug(f"{self.__class__}: Conditional: {value} {cond}{value2}")
             raise ValueError("Unknown value")
 
-        if not isinstance(value2, int) and not str.isdigit(value2):
+        if not isinstance(ivalue2, int) and not str.isdigit(value2):
             self.Logger.error(f"{self.__class__}: Unknown value: {value2} {ivalue2}")
             self.Logger.debug(f"{self.__class__}: Conditional: {value} {cond} {value2}")
             raise ValueError("Unknown value")
@@ -285,16 +292,30 @@ class BaseParser(object):
         else:
             return int(value, 10)
 
-    def PushConditional(self, v):
+    def PushConditional(self, v: bool, already_true: bool = False):
         """Push new value onto the conditional stack.
 
         Args:
-          v (obj): Value to push
+          v (bool): Value to push
+          already_true (bool): A boolean that specifies if this condition
+            (if/elseif/else) block has already been true.
+
+        !!! note
+            already_true is needed when calling PopConditional() to know if the next
+            part of the conditional block needs evaluated or not.
+
         """
-        self.ConditionalStack.append(v)
+        self.ConditionalStack.append((v, already_true))
 
     def PopConditional(self):
-        """Pops the current conditional and return the value."""
+        """Pops the current conditional and return the value.
+
+        Additionally returns a value specifying if the if/elseif/else block has already
+        returned true.  This is needed to know if the next part of the conditional block
+        needs evaluated or not.
+
+        Returns (bool, bool): (value, already_true)
+        """
         if (len(self.ConditionalStack) > 0):
             return self.ConditionalStack.pop()
         else:
@@ -314,10 +335,10 @@ class BaseParser(object):
         elif (v is None):
             return None
 
-        if (type(v) is bool):
+        if (isinstance(v, bool)):
             v = "true" if v else "false"
 
-        if (type(v) is str and (v.upper() == "TRUE" or v.upper() == "FALSE")):
+        if (isinstance(v, str) and (v.upper() == "TRUE" or v.upper() == "FALSE")):
             v = v.upper()
 
         return str(v)
@@ -377,7 +398,23 @@ class BaseParser(object):
         else:
             tokens = text.split()
         if (tokens[0].lower() == "!if"):
-            self.PushConditional(self.EvaluateConditional(text))
+            result = self.EvaluateConditional(text)
+            self.PushConditional(result, result)
+            return True
+
+        elif (tokens[0].lower() == "!elseif"):
+            if not self.InActiveCode():
+                (_, already_been_true) = self.PopConditional()
+
+                if already_been_true:
+                    self.PushConditional(False, True)
+                else:
+                    result = self.EvaluateConditional(text)
+                    self.PushConditional(result, result)
+            # already in active code, i.e. the previous if/elseif was true
+            else:
+                self.PopConditional()
+                self.PushConditional(False, True)
             return True
 
         elif (tokens[0].lower() == "!ifdef"):
@@ -398,9 +435,15 @@ class BaseParser(object):
             if len(tokens) != 1:
                 self.Logger.error("!ifdef conditionals need to be formatted correctly (spaces between each token)")
                 raise RuntimeError("Invalid conditional", text)
-            v = self.PopConditional()
+            (value, already_been_true) = self.PopConditional()
             # TODO make sure we can't do multiple else statements
-            self.PushConditional(not v)
+
+            # If we've already hit a true condition, we need to be false,
+            # otherwise, lets process!
+            if already_been_true:
+                self.PushConditional(False, True)
+            else:
+                self.PushConditional(True, True)
             return True
 
         elif (tokens[0].lower() == "!endif"):
@@ -415,9 +458,13 @@ class BaseParser(object):
     def EvaluateConditional(self, text):
         """Uses a pushdown resolver."""
         text = str(text).strip()
-        if not text.lower().startswith("!if "):
+        if text.lower().startswith("!if "):
+            text = text[3:].strip()
+        elif text.lower().startswith("!elseif "):
+            text = text[7:].strip()
+        else:
             raise RuntimeError(f"Invalid conditional cannot be validated: {text}")
-        text = text[3:].strip()
+
         self.Logger.debug(f"STAGE 1: {text}")
         text = self.ReplaceVariables(text)
         self.Logger.debug(f"STAGE 2: {text}")
@@ -606,7 +653,7 @@ class BaseParser(object):
 
     @classmethod
     def _IsOperator(cls, token):
-        if type(token) is not str:
+        if not isinstance(token, str):
             return False
         if token.startswith("!+"):
             token = token[2:]
@@ -633,7 +680,7 @@ class BaseParser(object):
             (bool): result of the state of the conditional you are in.
         """
         ret = True
-        for a in self.ConditionalStack:
+        for (a, _) in self.ConditionalStack:
             if not a:
                 ret = False
                 break
@@ -758,7 +805,32 @@ class HashFileParser(BaseParser):
         Args:
           line (str): line with a comment (#)
         """
-        return line.split('#')[0].strip()
+        if "#" not in line:
+            return line.strip()
+
+        result = []
+        inside_quotes = False
+        quote_char = None
+        escaped = False
+
+        for char in line:
+            if char in ('"', "'") and not escaped:
+                if not inside_quotes:
+                    inside_quotes = True
+                    quote_char = char
+                elif char == quote_char:
+                    inside_quotes = False
+                    quote_char = None
+            elif char == '#' and not inside_quotes:
+                break
+            elif char == '\\' and not escaped:
+                escaped = True
+            else:
+                escaped = False
+
+            result.append(char)
+
+        return ''.join(result).rstrip()
 
     def ParseNewSection(self, line):
         """Parses a new section line.
