@@ -12,8 +12,8 @@
 
 Some functionality copied from Tianocore/edk2 basetools.
 """
-import uuid
 import re
+import uuid
 
 
 class GuidParser():
@@ -99,7 +99,7 @@ class GuidParser():
                 int(guidValueList[9], 16),
                 int(guidValueList[10], 16)
             )
-        except:
+        except Exception:
             return ''
 
     @classmethod
