@@ -6,12 +6,14 @@
 # SPDX-License-Identifier: BSD-2-Clause-Patent
 ##
 """Code to help parse EDK2 INF files."""
-from edk2toollib.uefi.edk2.parsers.base_parser import HashFileParser
 import os
+import re
 
+from edk2toollib.uefi.edk2.parsers.base_parser import HashFileParser
 
 AllPhases = ["SEC", "PEIM", "PEI_CORE", "DXE_DRIVER", "DXE_CORE", "DXE_RUNTIME_DRIVER", "UEFI_DRIVER",
-             "SMM_CORE", "DXE_SMM_DRIVER", "UEFI_APPLICATION"]
+             "SMM_CORE", "DXE_SMM_DRIVER", "UEFI_APPLICATION", "MM_STANDALONE", "MM_CORE_STANDALONE",
+             "HOST_APPLICATION",]
 
 
 class InfParser(HashFileParser):
@@ -35,6 +37,10 @@ class InfParser(HashFileParser):
 
     NOTE: Key / Value pairs determined by lines that contain a single =
     """
+    SECTION_REGEX = re.compile(r"\[(.*)\]")
+    SECTION_LIBRARY = re.compile(r'libraryclasses(?:\.([^,.\]]+))?[,.\]]', re.IGNORECASE)
+    SECTION_SOURCE = re.compile(r'sources(?:\.([^,.\]]+))?[,.\]]', re.IGNORECASE)
+
     def __init__(self):
         """Inits an empty parser."""
         HashFileParser.__init__(self, 'ModuleInfParser')
@@ -45,6 +51,8 @@ class InfParser(HashFileParser):
         self.SupportedPhases = []
         self.PackagesUsed = []
         self.LibrariesUsed = []
+        self.ScopedLibraryDict = {}
+        self.ScopedSourceDict = {}
         self.ProtocolsUsed = []
         self.GuidsUsed = []
         self.PpisUsed = []
@@ -53,6 +61,22 @@ class InfParser(HashFileParser):
         self.Binaries = []
         self.Path = ""
 
+    def get_libraries(self, arch_list: list[str]):
+        """Returns a list of libraries depending on the requested archs."""
+        libraries = self.ScopedLibraryDict.get("common", []).copy()
+
+        for arch in arch_list:
+            libraries = libraries + self.ScopedLibraryDict.get(arch.lower(), []).copy()
+        return list(set(libraries))
+
+    def get_sources(self, arch_list: list[str]):
+        """Returns a list of sources depending on the requested archs."""
+        sources = self.ScopedSourceDict.get("common", []).copy()
+
+        for arch in arch_list:
+            sources = sources + self.ScopedSourceDict.get(arch.lower(), []).copy()
+        return list(set(sources))
+
     def ParseFile(self, filepath):
         """Parses the INF file provided."""
         self.Logger.debug("Parsing file: %s" % filepath)
@@ -80,7 +104,15 @@ class InfParser(HashFileParser):
             if (sline is None or len(sline) < 1):
                 continue
 
-            if InDefinesSection:
+            sline = self.ReplaceVariables(sline)
+
+            if sline.strip().startswith("DEFINE"):
+                tokens = sline.strip().replace("DEFINE", "").split('=', 1)
+                self.LocalVars[tokens[0].strip()] = tokens[1].strip()
+                self.Logger.info(f"Key,values found for local vars: {tokens[0].strip()}, {tokens[1].strip()}")
+                continue
+
+            elif InDefinesSection:
                 if sline.strip()[0] == '[':
                     InDefinesSection = False
                 else:
@@ -191,4 +223,61 @@ class InfParser(HashFileParser):
             elif sline.strip().lower().startswith('[binaries'):
                 InBinariesSection = True
 
+        # Re-parse for scoped information
+        # Hopefully eventually replace all of the above with this
+        # logic
+        current_section = ""
+        arch_list = []
+
+        for line in self.Lines:
+            line = self.StripComment(line)
+
+            if line.strip() == "":
+                continue
+
+            if line.strip().startswith("#"):
+                continue
+
+            if line.strip().startswith("DEFINE"):
+                tokens = line.strip().replace("DEFINE", "").split('=', 1)
+                self.LocalVars[tokens[0].strip()] = tokens[1].strip()
+                self.Logger.info(f"Key,values found for local vars: {tokens[0].strip()}, {tokens[1].strip()}")
+                continue
+
+            line = self.ReplaceVariables(line)
+
+            match = self.SECTION_LIBRARY.findall(line)
+            if match:
+                current_section = "LibraryClasses"
+                arch_list = match
+                continue
+            match = self.SECTION_SOURCE.findall(line)
+            if match:
+                current_section = "Sources"
+                arch_list = match
+                continue
+
+            # Catch all others
+            match = self.SECTION_REGEX.match(line)
+            if match:
+                current_section = match.group(1)
+                arch_list = []
+
+            # Handle lines when we are in a library section
+            if current_section == "LibraryClasses":
+                for arch in arch_list:
+                    arch = 'common' if arch == '' else arch.lower()
+                    if arch in self.ScopedLibraryDict:
+                        self.ScopedLibraryDict[arch].append(line.split()[0].strip())
+                    else:
+                        self.ScopedLibraryDict[arch] = [line.split()[0].strip()]
+
+            # Handle lines when we are in a source section
+            if current_section == "Sources":
+                for arch in arch_list:
+                    arch = 'common' if arch == '' else arch.lower()
+                    if arch in self.ScopedSourceDict:
+                        self.ScopedSourceDict[arch].append(line.split()[0].strip())
+                    else:
+                        self.ScopedSourceDict[arch] = [line.split()[0].strip()]
         self.Parsed = True
