@@ -8,8 +8,8 @@
 # SPDX-License-Identifier: BSD-2-Clause-Patent
 ##
 """Contains classes to help with parsing INF files that may contain OVERRIDE information."""
-import os
 import datetime
+import os
 
 FORMAT_VERSION_1 = (1, 4)   # Version 1: #OVERRIDE : VERSION | PATH_TO_MODULE | HASH | YYYY-MM-DDThh-mm-ss
 
@@ -109,7 +109,7 @@ class OverrideParser(object):
         parse_lines = parse_contents.split('\n')
         result = []
 
-        for i in range(0, len(parse_lines)):
+        for i in range(len(parse_lines)):
             if parse_lines[i].strip().upper().startswith("#OVERRIDE"):
                 result.append({'lineno': i + 1, 'line': parse_lines[i].strip()})
 
