@@ -7,22 +7,21 @@
 # SPDX-License-Identifier: BSD-2-Clause-Patent
 ##
 """Module containing utility functions to support re-use in python scripts."""
-import re
-import os
-import stat
-import logging
 import datetime
-import time
+import importlib
+import inspect
+import locale
+import logging
+import os
+import platform
+import re
 import shutil
-import threading
+import stat
 import subprocess
 import sys
-import inspect
-import platform
-import importlib
+import threading
+import time
 from collections import namedtuple
-import locale
-
 from warnings import warn
 
 
@@ -71,7 +70,7 @@ def reader(filepath, outstream, stream, logging_level=logging.INFO, encodingErro
 
     (_, encoding) = locale.getdefaultlocale()
     while True:
-        s = stream.readline().decode(encoding, errors=encodingErrors)
+        s = stream.readline().decode(encoding or 'utf-8', errors=encodingErrors)
         if not s:
             break
         if (f is not None):
@@ -392,15 +391,19 @@ def RemoveTree(dir_path: str, ignore_errors: bool = False) -> None:
         dir_path (str): Path to directory to remove.
         ignore_errors (bool): ignore errors during removal
     """
-    # TODO actually make this private with _
-    def remove_readonly(func, path, _):
+    # Allows for finding paths over the 260 character limit, even if the registry key (LongPathsEnabled)
+    # Is not enabled.
+    if sys.platform.startswith('win'):
+        dir_path = '\\\\?\\' + os.path.abspath(dir_path)
+
+    def _remove_readonly(func, path, _):
         """Private function to attempt to change permissions on file/folder being deleted."""
         os.chmod(path, stat.S_IWRITE)
         func(path)
 
     for _ in range(3):  # retry up to 3 times
         try:
-            shutil.rmtree(dir_path, ignore_errors=ignore_errors, onerror=remove_readonly)
+            shutil.rmtree(dir_path, ignore_errors=ignore_errors, onerror=_remove_readonly)
         except OSError as err:
             logging.warning(f"Failed to fully remove {dir_path}: {err}")
         else:
@@ -411,7 +414,7 @@ def RemoveTree(dir_path: str, ignore_errors: bool = False) -> None:
 
 def PrintByteList(ByteList, IncludeAscii=True, IncludeOffset=True, IncludeHexSep=True, OffsetStart=0, **kwargs):
     """Print a byte array as hex and optionally output ascii as well as offset within the buffer."""
-    out_fs = kwargs.get("out_fs", sys.stdout)
+    outfs = kwargs.get("outfs", sys.stdout)
     kwargs["include_ascii"] = IncludeAscii
     kwargs["include_hex_sep"] = IncludeHexSep
     kwargs["include_offset"] = IncludeOffset
@@ -420,16 +423,17 @@ def PrintByteList(ByteList, IncludeAscii=True, IncludeOffset=True, IncludeHexSep
         "This function is being replaced by hexdump, if you rely on this behavior switch to hexdump",
         DeprecationWarning
     )
-    hexdump(ByteList, offset_start=OffsetStart, out_fs=out_fs, **kwargs)
+    hexdump(ByteList, offset_start=OffsetStart, outfs=outfs, **kwargs)
 
 
-def hexdump(byte_list, offset_start=0, out_fs=sys.stdout, **kwargs) -> None:
+def hexdump(byte_list, offset_start=0, outfs=sys.stdout, **kwargs) -> None:
     """Print a byte array as hex and optionally output ascii as well as offset within the buffer.
 
     Args:
         byte_list (bytearray): byte array to print
         offset_start (int): offset to print to the side of the hexdump
-        out_fs (io.BytesIO): output file stream to print to
+        outfs (io.BytesIO): output file stream to print to
+        kwargs (any): keyword arguments expanded below.
 
     Keyword Arguments:
         include_ascii (bool): Option (Default: True) to include ascii
@@ -449,14 +453,14 @@ def hexdump(byte_list, offset_start=0, out_fs=sys.stdout, **kwargs) -> None:
     for index, byte in enumerate(byte_list):
         # Start of New Line
         if index % 16 == 0 and include_offset:
-            out_fs.write(f"{(index + offset_start):#04x} -")
+            outfs.write(f"{(index + offset_start):#04x} -")
 
         # Midpoint of a Line
         if index % 16 == 8 and include_hex_sep:
-            out_fs.write(" -")
+            outfs.write(" -")
 
         # Print As Hex Byte
-        out_fs.write(f" {byte:#04x}")
+        outfs.write(f" {byte:#04x}")
 
         # Prepare to Print As Ascii
         if byte < 0x20 or byte > 0x7E:
@@ -467,9 +471,9 @@ def hexdump(byte_list, offset_start=0, out_fs=sys.stdout, **kwargs) -> None:
         # End of Line
         if index % 16 == 15:
             if include_ascii:
-                out_fs.write(f" {ascii_string} ")
+                outfs.write(f" {ascii_string} ")
             ascii_string = ""
-            out_fs.write("\n")
+            outfs.write("\n")
 
     # Done - Lets check if we have partial
     if index % 16 != 15:
@@ -478,15 +482,15 @@ def hexdump(byte_list, offset_start=0, out_fs=sys.stdout, **kwargs) -> None:
             # Pad out to the correct spot
 
             while index % 16 != 15:
-                out_fs.write("     ")
+                outfs.write("     ")
                 if index % 16 == 7:  # account for the - symbol in the hex dump
                     if include_offset:
-                        out_fs.write("  ")
+                        outfs.write("  ")
                 index += 1
             # print the ascii partial line
-            out_fs.write(f" {ascii_string} ")
-            # print a single newline so that next print will be on new line
-        out_fs.write("\n")
+            outfs.write(f" {ascii_string} ")
+        # print a single newline so that next print will be on new line
+        outfs.write("\n")
 
 
 def export_c_type_array(buffer_fs, variable_name, out_fs, **kwargs) -> None:
@@ -496,6 +500,7 @@ def export_c_type_array(buffer_fs, variable_name, out_fs, **kwargs) -> None:
         buffer_fs (io.BytesIO): buffer file stream to turn into a C style array
         variable_name (str): variable name to use for the C style array
         out_fs (io.StringIO): output filestream to write to
+        kwargs (any): keyword arguments expanded below.
 
     Keyword Arguments:
         data_type (str): The datatype of the array (Default: UINT8)
@@ -507,6 +512,7 @@ def export_c_type_array(buffer_fs, variable_name, out_fs, **kwargs) -> None:
         include_ascii (bool): includes a ascii comment to side of hex
         include_length (bool): includes length in the decleration of array
             (ex. "UINT8 TestVariable[13] = {")
+
     Return:
          None
 
