@@ -10,8 +10,8 @@
 
 Supports targeting multiple ESRT nodes with the same INF
 """
-import textwrap
 import re
+import textwrap
 import uuid
 from datetime import datetime
 from typing import Optional
@@ -121,7 +121,7 @@ class InfHeader(object):
             CatalogFile={self.Name}.cat
 
             [Manufacturer]
-            %MfgName% = Firmware,NT{self.Arch}
+            %MfgName% = Firmware,NT{self.Arch}.10.0...16299
 
             """)
 
@@ -215,7 +215,7 @@ class InfFirmware(object):
         # build integrity file string, if required.
         if (self.IntegrityFile is not None):
             IntegrityFile = f"{self.IntegrityFile}\n"
-            IntegrityFileReg = f"HKR,,FirmwareIntegrityFilename,,{self.IntegrityFile}\n"
+            IntegrityFileReg = f"HKR,,FirmwareIntegrityFilename,,%13%\\{self.IntegrityFile}\n"
         else:
             IntegrityFile = ""
             IntegrityFileReg = ""
@@ -237,7 +237,7 @@ class InfFirmware(object):
             [{self.Tag}_AddReg]
             HKR,,FirmwareId,,{{{self.EsrtGuid}}}
             HKR,,FirmwareVersion,%REG_DWORD%,0x{self.VersionInt:X}
-            HKR,,FirmwareFilename,,{self.FirmwareFile}
+            HKR,,FirmwareFilename,,%13%\\{self.FirmwareFile}
             """)
         outstr += IntegrityFileReg + "\n"
         return outstr
@@ -284,7 +284,7 @@ class InfFirmwareSections(object):
 
         This includes any InfFirmware objects in it.
         """
-        firmwareStr = f"[Firmware.NT{self.Arch}]\n"
+        firmwareStr = f"[Firmware.NT{self.Arch}.10.0...16299]\n"
         for InfFirmware in self.Sections.values():
             firmwareStr += f"%{InfFirmware.Tag}Desc% = {InfFirmware.Tag}_Install,UEFI\\RES_{{{InfFirmware.EsrtGuid}}}\n"
         firmwareStr += "\n"
@@ -331,7 +331,7 @@ class InfSourceFiles(object):
         outstr += Files
         outstr += textwrap.dedent("""
             [DestinationDirs]
-            DefaultDestDir = %DIRID_WINDOWS%,Firmware ; %SystemRoot%\\Firmware
+            DefaultDestDir = 13
 
             """)
         return outstr
